import greenfoot.*;
import java.lang.reflect.*;


public abstract class AbstractBullet extends MapActor
{
    private double speed;
    private int damage;
    private AbstractVirus target;
    private AbstractTower holder;
    private Color particleColor;
    
    public AbstractBullet(AbstractTower towerOfBullet, AbstractVirus target, Color particleColor, double speed) {
        CybervirusHazard.getCurrentMapView().add(this, towerOfBullet.getLocation());
        this.holder = towerOfBullet;
        this.target = target;
        this.damage = towerOfBullet.getParameters().getDamage();
        GifImage image = ImageLoader.get("map.sprite.bullet." + towerOfBullet.getName());
        image.setZoom(2);
        setImage(image);
        this.speed = speed;
        this.particleColor = particleColor;
    }
    
    public void act() {
        super.act();
        if(target.died()) { 
            suicide();
            return;
        }
        if(intersects(target)) {
            onVirusReached(target);
            target.performDamage(damage);
            suicide();
            return;
        }
        setRotation((int) Math.round(Math.atan2(target.getY() - this.getY(), target.getX() - this.getX()) * 180.0 / Math.PI));
        move(speed);
        ParticleSystem.create(getLocation().add(new Location((int) (Math.random() * 4 - 2), (int) (Math.random() * 4 - 2))), 2, particleColor, 2, 0L, 200L, 0.0, -0.2, 0.2, -0.2, 0.2);
        this.speed *= 1.05;
    }
    
    public AbstractTower getHolder() {
        return holder;
    }
    
    public abstract void onVirusReached(AbstractVirus virus);
}
