import greenfoot.*;
import java.util.*;


public class AbstractVirus extends CompositeMapActor
{
    private String name;
    private final int maxHealth;
    private int health;
    private int energyRewardOnDeath;
    private double speed;
    private HealthBar healthBar;
    private int virusPathCornerMovingTo = 1;
    private ArrayList<VirusParameterModifier> parameterModifiers = new ArrayList<>();
    
    public AbstractVirus(String name, int maxHealth, int energyRewardOnDeath, double speed) {
        this.name = name;
        this.maxHealth = maxHealth;
        this.health = maxHealth;
        this.speed = speed;
        this.energyRewardOnDeath = energyRewardOnDeath;
        this.setImage(ImageLoader.get("map.sprite.virus." + name.toLowerCase()));
        this.healthBar = new HealthBar(this);
    }
    
    public void act() {
        super.act();
        Location virusPathCorner = CybervirusHazard.getCurrentMapView().getMap().getVirusPath().getNullActors().get(virusPathCornerMovingTo).getLocation();
        if(new Location(virusPathCorner.getX() / 2, virusPathCorner.getY() / 2).distanceTo(getLocation()) < speed) {
            setLocation(new Location(virusPathCorner.getX() / 2, virusPathCorner.getY() / 2));
            if(CybervirusHazard.getCurrentMapView().getMap().getVirusPath().getNullActors().size() - 1 <= virusPathCornerMovingTo) {
                killNotRewarded();
                CybervirusHazard.getCurrentMapView().getMap().getVirusPath().getFileStorage().hit();
                return;
            } else {
                virusPathCornerMovingTo++;
            }
        } else {
            turnTowards((int) (virusPathCorner.getX() / 2), (int) (virusPathCorner.getY() / 2));
            double speedAfterModifications = speed;
            for(VirusParameterModifier modifier : parameterModifiers) speedAfterModifications *= modifier.getSpeedMultiplier();
            move(speedAfterModifications < speed / 8 ? speed / 8 : speedAfterModifications);
        }
    }
    
    public void performDamage(int damage) {
        this.health -= damage;
        if(health <= 0) suicide();
    }
    
    public int getHealth() {
        return health;
    }
    
    public int getMaxHealth() {
        return maxHealth;
    }
    
    public void killNotRewarded() {
        super.kill();
    }
    
    @Override
    public void kill() {
        super.kill();
        CybervirusHazard.getCurrentMapView().addEnergy(energyRewardOnDeath * (1 + CybervirusHazard.getCurrentMapView().getMap().getWaveManager().getCurrentWaveIndex()));
    }
    
    public Iterable<CompanionActorHolder> getCompanions() { 
        return List.of(
            new CompanionActorHolder(healthBar, new Location(0, -8 - HealthBar.getHealthBarsDistanceToViruses()))
        ); 
    }
    
    public int getEnergyRewardOnDeath() {
        return energyRewardOnDeath;
    }
    
    public void addModifier(VirusParameterModifier modifier) {
        parameterModifiers.add(modifier);
    }
}
