import greenfoot.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;


public class Label extends InterfaceElement {
    private String text;
    private int textWidth;
    private GreenfootImage image;
    private Font font;
    private Color background;
    private Color foreground;
    private LabelOrientation orientation = LabelOrientation.LEFT;
    
    public Label(Object text, String fontName, int fontSize, Color foreground, Color background) {
        this.font = new Font(fontName, fontSize);
        this.foreground = foreground;
        this.background = background;
        setText(text);
    }
    
    public Label(Object text, String fontName, int fontSize, Color foreground) {
        this(text, fontName, fontSize, foreground, new Color(0, 0, 0, 0));
    }
    
        
    public Label(Object text, String fontName, int fontSize) {
        this(text, fontName, fontSize, Colors.VERY_LIGHT_GRAY);
    }
    
    public Label(Object text, int fontSize, Color foreground) {
        this(text, CybervirusHazard.getFontName(), fontSize, foreground);
    }
    
    public Label(Object text, int fontSize) {
        this(text, CybervirusHazard.getFontName(), fontSize, Colors.VERY_LIGHT_GRAY);
    }
    
    public Label(Object text, String fontName) { 
        this(text, fontName, 16);
    }
    
    public Label(Object text) {
        this(text, CybervirusHazard.getFontName());
    }
    
    public Label() {
        this("");
    }
    
    public void setOrientation(LabelOrientation orientation) {
        this.orientation = orientation;
        setText(text);
    }
    
    public void setText(Object object) {
        this.text = object.toString();
        java.awt.Font awtFont = new java.awt.Font(font.getName(), java.awt.Font.PLAIN, font.getSize());
        FontRenderContext renderContext = new FontRenderContext(new AffineTransform(), true, true);
        Rectangle2D renderedFontRectangle = awtFont.getStringBounds(text, renderContext);
        this.image = new GreenfootImage((int) Math.ceil(renderedFontRectangle.getWidth() + 1) * 2, (int) Math.ceil(renderedFontRectangle.getHeight() + 2));
        this.image.setFont(font);
        this.image.setColor(background);
        this.image.fill();
        this.image.setColor(foreground);
        this.textWidth = (int) renderedFontRectangle.getWidth();
        int x = 0;
        switch(orientation) {
            case LEFT:
                x = textWidth;
                break;
            
            case RIGHT:
                x = 0;
                break;
            
            case CENTER:
                x = textWidth / 2;
                break;
            default:
                throw new UnsupportedOperationException("Unable to set label orientation to " + orientation.toString().toLowerCase());
        }
        this.image.drawString(text, x, (int) Math.ceil(renderedFontRectangle.getHeight()));
        this.setImage(image);
    }
    
    @Override
    public void kill() {
        super.kill();
        setText("");
    }
    
    public String getText() {
        return text;
    }
    
    public void act() {
        super.act();
        setImage(image);
    }
    
    public int getTextWidth() {
        return textWidth;
    }
}