import greenfoot.*;
import java.util.*;


public class MapActor extends SmoothMover {
    private static final ArrayList<MapActor> createdActors = new ArrayList<>();
    private static final double SHAKE_POWER_REDUCTION_COEFFICIENT = 0.0008;
    private static double shakePower = 0.0;
    private static int shakeShiftX = 0;
    private static int shakeShiftY = 0;
    private GifImage image = null;
    boolean neverActed = true;
    
    public MapActor(String imageLoaderKey) {
        this(ImageLoader.get(imageLoaderKey));
    }
    
    public MapActor(GifImage image) {
        this.image = image;
        setImage(image.getCurrentImage());
        createdActors.add(this);
    }
    
    public MapActor(GreenfootImage image) {
        this(new GifImage(image));
    }
    
    public MapActor() {
        this(GifImage.createEmpty());
    }
    
    public void setZoom(double zoom) {
        image.setZoom(zoom);
    }
    
    public boolean alive() {
        return getWorld() != null;
    }
    
    public boolean died() {
        return !alive();
    }
    
    public void kill() {
        if(died()) return;
        onDeath();
        getWorld().removeObject(this);
        createdActors.remove(this);
    }
    
    public void suicide() { kill(); }
    
    public void onDeath() {}
    
    public void onSpawn() {}
    
    public void onClick() {}
    
    public void act() {
        if(image != null) setImage(image.getCurrentImage());
        if(!neverActed) {
            onSpawn();
            this.neverActed = false;
        }
        if(Greenfoot.mouseClicked(this)) onClick();
        
        if(shakePower != 0) {
            int nextShakeShiftX = (int) ((Math.random() * 2 - 1) * shakePower);
            int nextShakeShiftY = (int) ((Math.random() * 2 - 1) * shakePower);
            for(MapActor actor : createdActors) {
                if(actor.alive()) actor.setLocation(actor.getExactX() - shakeShiftX + nextShakeShiftX, actor.getExactY() - shakeShiftY + nextShakeShiftY);
            }
            shakeShiftX = nextShakeShiftX;
            shakeShiftY = nextShakeShiftY;
            
            shakePower = shakePower <= 1 ? SHAKE_POWER_REDUCTION_COEFFICIENT : shakePower - SHAKE_POWER_REDUCTION_COEFFICIENT;
            // System.out.println(shakeShiftX + " " + shakeShiftY);
        }
    }
    
    public void setImage(GifImage image) { this.image = image; }
    
    public void setImage(String key) { setImage(ImageLoader.get(key)); }
    
    public GifImage getGifImage() { return image; }
    
    public Location getLocation() { return alive() ? new Location(getX(), getY()) : new Location(-1, -1); }
    
    public void setLocation(Location location) {
        super.setLocation(location.getX(), location.getY());
    }
    
    public <T extends Actor> List<T> inRange(int range, Class<T> type) {
        return super.getObjectsInRange(range, type);
    }
    
    public static ArrayList<MapActor> getActors() {
        return createdActors;
    }
    
    public static void addShakePower(double extraPower) {
        shakePower += extraPower;
    }
    
    public static double getShakeShiftX() {
        return shakeShiftX;
    }
    
    public static double getShakeShiftY() {
        return shakeShiftY;
    }
}
