import javafx.scene.paint.Color;
import greenfoot.*;
import java.util.List;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.BasicStroke;


public class MapView extends AbstractView
{
    private Map map;
    private int energy = 1_000_000_000;
    private boolean isToastAboutWaveIsComingShown = false;
    
    public MapView() {
        super();
        fill(Colors.VERY_DARK_GRASS.darker().darker().darker());
        CybervirusHazard.init();
        
        map = new Map(new VirusPath(this, new Location(CybervirusHazard.getViewsWidth() * 2, 100),
                                                  new Location(CybervirusHazard.getViewsWidth(), 100),
                                                  new Location(CybervirusHazard.getViewsWidth(), CybervirusHazard.getViewsHeight() * 2 - 100),
                                                  new Location(100, CybervirusHazard.getViewsHeight() * 2 - 100)), 
                                                  
                                                  new WaveManager(
                                                        new Wave("Common", 5, 2000),
                                                        new Wave("Common", 10, 1400),
                                                        new Wave("Strong", 15, 1300),
                                                        new Wave("Fast", 15, 1250),
                                                        new Wave("Strong", 25, 1200),
                                                        new Wave("Common", 50, 1000),
                                                        new Wave("Strong", 50, 2000),
                                                        new Wave("Common", 100, 1500),
                                                        new Wave("Fast", 150, 1400),
                                                        new Wave("Fast", 200, 1400),
                                                        new Wave("Common", 300, 1400),
                                                        new Wave("Strong", 500, 1300),
                                                        new Wave("Strong", 500, 1200),
                                                        new Wave("Strong", 500, 1100)
                                                  ),
                                                  
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 - 50, CybervirusHazard.getViewsHeight() / 2),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 + 50, CybervirusHazard.getViewsHeight() / 2),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 - 50, CybervirusHazard.getViewsHeight() / 2 + 80),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 + 50, CybervirusHazard.getViewsHeight() / 2 + 80),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 - 50, CybervirusHazard.getViewsHeight() / 2 - 80),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 + 50, CybervirusHazard.getViewsHeight() / 2 - 80),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 - 50, CybervirusHazard.getViewsHeight() / 2 + 40),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 + 50, CybervirusHazard.getViewsHeight() / 2 + 40),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 - 50, CybervirusHazard.getViewsHeight() / 2 - 40),
                                                  new Location(CybervirusHazard.getViewsWidth() / 2 + 50, CybervirusHazard.getViewsHeight() / 2 - 40)
                                                  /*new Location(50, 100),
                                                  new Location(100, 100),
                                                  new Location(150, 100),
                                                  new Location(200, 100),
                                                  new Location(250, 100),
                                                  
                                                  new Location(50, 150),
                                                  new Location(100, 150),
                                                  new Location(150, 150),
                                                  new Location(200, 150),
                                                  new Location(250, 150)*/
                                                  );
        
        setPaintOrder(Icon.class,
                      Label.class,
                      InterfaceElement.class,
                      CompositeInterfaceElement.class,
                      ScreenFogger.class,
                      HeaderBar.class, 
                      TowerSlot.class, 
                      AbstractTower.class,
                      AbstractEffect.class,  
                      TowerRadiusVisualizer.class, 
                      HealthBar.class, 
                      AbstractVirus.class, 
                      MapActor.class,
                      Actor.class);
        
        GreenfootImage logo = ImageLoader.get("map.background.logo").getCurrentImage();
        logo.scale(logo.getWidth() * 6, logo.getHeight() * 6);
        logo.setTransparency(45);
        getBackground().drawImage(logo, CybervirusHazard.getViewsWidth() - logo.getWidth() / 2, CybervirusHazard.getViewsHeight() - logo.getHeight() / 2);
        
        for(Location slotLocation : map.getSlotLocations()) {
            add(new TowerSlot(), slotLocation);
        }
        
        getBackground().setColor(Colors.VERY_DARK_GRASS.darker());
        
        for(int i = 1; i < map.getVirusPath().getNullActors().size(); i++) {
            Graphics2D g2d = ((Graphics2D) getBackground().getAwtImage().getGraphics());
            g2d.setColor(new java.awt.Color(Colors.DARK_GRASS.getRed(), Colors.DARK_GRASS.getGreen(), Colors.DARK_GRASS.getBlue()));
            g2d.setStroke(new BasicStroke(2, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0, new float[]{2, 4}, 3));
            g2d.drawLine(map.getVirusPath().getNullActors().get(i - 1).getX(),
                                     map.getVirusPath().getNullActors().get(i - 1).getY(),
                                     map.getVirusPath().getNullActors().get(i).getX(),
                                     map.getVirusPath().getNullActors().get(i).getY());
        }
        addComposite(new HeaderBar(), CybervirusHazard.getViewsWidth() / 2, 8);
    }
    
    public void add(MapActor actor, Location location) {
        this.addObject(actor, (int) (location.getX() + MapActor.getShakeShiftX()), (int) (location.getY() + MapActor.getShakeShiftY()));
        actor.setLocation(location.getX() + MapActor.getShakeShiftX(),
                          location.getY() + MapActor.getShakeShiftY());
    }
    
    public void add(MapActor actor, double x, double y) {
        add(actor, new Location(x, y));
    }
    
    public void addComposite(CompositeMapActor actor, Location location) {
        add(actor, location);
        for(CompanionActorHolder companion : actor.getCompanions()) {
            add(companion.toActor(), location.add(companion.getRelativeLocation()));
        }
    }
    
    public void addComposite(CompositeMapActor actor, double x, double y) {
        addComposite(actor, new Location(x, y));
    }
    
    public int getEnergy() {
        return this.energy;
    }
    
    public void addEnergy(int amount) {
        this.energy += amount;
    }
    
    public void removeEnergy(int amount) {
        addEnergy(-amount);
    }
    
    public boolean hasEnoughEnergy(int energy) {
        return this.energy >= energy;
    }
    
    public void act() {
        if(map.getWaveManager().isVirusShouldBeGenerated()) {
            addComposite(map.getWaveManager().createVirus(), map.getVirusPath().getVirusSpawnLocation().getX() / 2, map.getVirusPath().getVirusSpawnLocation().getY() / 2);
        }
        if(map.getWaveManager().isCurrentWaveOver() && !map.getWaveManager().isCurrentWaveLast() && !map.getWaveManager().isTimeBetweenWaves()) {
            map.getWaveManager().nextWave();
            Toast.show("wave", "Wave has been started!", 8000, Colors.LEMONADE);
            isToastAboutWaveIsComingShown = false;
        } else {
            if(!isToastAboutWaveIsComingShown) Toast.show("wave", "Wave will be activated in " + WaveManager.getDelayBetweenWaves() / 1000 + " seconds!", WaveManager.getDelayBetweenWaves() / 2, Colors.LEMONADE);
            isToastAboutWaveIsComingShown = true;
        }
    }
    
    public Map getMap() {
        return map;
    }
    
    public void shake(double power) {
        MapActor.addShakePower(power);
    }
}
