import greenfoot.*;


public class Particle extends MapActor {
    private double gravitation;
    private double velocityX;
    private double velocityY;
    private double sizeReduction;
    private double size = 1.0;
    private long createdAt;
    private long lifetime;
    
        
    public Particle(GifImage image, long lifetime, double gravitation, double velocityX, double velocityY, double sizeReduction) {
        super(image);
        this.gravitation = gravitation;
        this.velocityX = velocityX;
        this.velocityY = velocityY;
        this.createdAt = System.currentTimeMillis();
        this.lifetime = lifetime;
        this.sizeReduction = sizeReduction;
        
        setRotation((int) Math.toDegrees(Math.atan(velocityY / velocityX)) + (velocityX < 0 ? 180 : 0));
    }
    
    public Particle(GreenfootImage image, long lifetime, double gravitation, double velocityX, double velocityY, double sizeReduction) {
        this(new GifImage(image), lifetime, gravitation, velocityX, velocityY, sizeReduction);
    }
    
    public Particle(String particleKey, long lifetime, double gravitation, double velocityX, double velocityY, double sizeReduction) {
        this(ImageLoader.get("map.sprite.fx.particle." + particleKey), lifetime, gravitation, velocityX, velocityY, sizeReduction);
    }
    
    public Particle(Color color, int size, long lifetime, double gravitation, double velocityX, double velocityY, double sizeReduction) {
        this(Images.create(color, size), lifetime, gravitation, velocityX, velocityY, sizeReduction);
    }
    
    public Particle(GreenfootImage image, long lifetime, double gravitation, double velocityX, double velocityY) {
        this(new GifImage(image), lifetime, gravitation, velocityX, velocityY, 1);
    }
    
    public Particle(String particleKey, long lifetime, double gravitation, double velocityX, double velocityY) {
        this(ImageLoader.get("map.sprite.fx.particle." + particleKey), lifetime, gravitation, velocityX, velocityY, 1);
    }
    
    public Particle(Color color, int size, long lifetime, double gravitation, double velocityX, double velocityY) {
        this(Images.create(color, size), lifetime, gravitation, velocityX, velocityY, 1);
    }
    
    public void act() {
        super.act();
        if(createdAt + lifetime <= System.currentTimeMillis()) suicide();
        setRotation((int) Math.toDegrees(Math.atan(velocityY / velocityX)) + (velocityX < 0 ? 180 : 0));
        move(Math.sqrt(Math.pow(velocityX, 2) + Math.pow(velocityY, 2)));
        velocityY += gravitation;
        if(Math.random() < .3) {
            size *= sizeReduction;
            setZoom(size);
        }
    }
}
