import greenfoot.*;
import java.util.*;


public class Toast extends CompositeInterfaceElement
{
    private static ArrayList<Toast> addedToasts = new ArrayList<>();
    private static final int DISTANCE_BETWEEN_TOASTS = 0;
    private static final int TOAST_WIDTH = 200;
    private static final int TOAST_HEIGHT = 20;
    private static final long TRANSITION_LENGTH_MILLIS = 800;
    private static final double ANIMATION_EASING_COEFFICIENT = .40;
    private Icon icon;
    private Label title;
    private String id;
    private long birthtime;
    private long lifetime;
    
    public static void show(String icon, String title, long lifetime, Color foreground) {
        int firstNull = addedToasts.indexOf(null); 
        Toast toast = new Toast(icon, title, lifetime, foreground);
        if(firstNull == -1) addedToasts.add(toast); else addedToasts.set(firstNull, toast);
        CybervirusHazard.getCurrentMapView().addComposite(toast, new Location(-TOAST_WIDTH / 2, 20 + (firstNull == -1 ? addedToasts.size() - 1 : firstNull) * 10));
    }
    
    public static void show(String icon, String title, long lifetime) {
        show(icon, title, lifetime, Colors.VERY_LIGHT_GRAY);
    }
    
    private Toast(String icon, String title, long lifetime, Color foreground) {
        this.icon = new Icon("toast." + icon);
        this.title = new Label(title, 10, foreground);
        this.birthtime = System.currentTimeMillis();
        this.lifetime = lifetime;
        this.setImage(Images.create(new Color(0, 0, 0), TOAST_WIDTH, TOAST_HEIGHT));
    }
    
    private Toast(String icon, String title, long lifetime) {
        this(icon, title, lifetime, Colors.VERY_LIGHT_GRAY);
    }
    
    public void act() {
        super.act();
        if(birthtime + lifetime <= System.currentTimeMillis()) {
            int index = addedToasts.indexOf(this);
            if(index != -1) addedToasts.set(index, null);
            suicide();
            return;
        }
        if(System.currentTimeMillis() - birthtime <= TRANSITION_LENGTH_MILLIS) {
            ((MapActor) this).setRotation(0);
            move(Math.abs(-getX() + 10) * ANIMATION_EASING_COEFFICIENT);
        } else if(birthtime + lifetime - System.currentTimeMillis() <= TRANSITION_LENGTH_MILLIS) {
            ((MapActor) this).setRotation(180);
            move(Math.abs(-getX() - (title.getTextWidth() + 60) / 2) * ANIMATION_EASING_COEFFICIENT);
        }
    }    
    
    public Iterable<CompanionActorHolder> getCompanions() {
        return List.of(
            new CompanionActorHolder(icon, new Location(4, 4)),
            new CompanionActorHolder(title, new Location(24, 4))
        );
    }
}
