import java.util.List;
import java.util.ArrayList;


public class WaveManager {
    private List<Wave> waves = new ArrayList<>();
    private int currentWave = 0;
    private long lastWaveOverTime = 0;
    
    private static final long DELAY_BETWEEN_WAVES = 5000;
    
    
    public WaveManager(Wave... waves) {
        if(waves.length == 0) throw new UnsupportedOperationException("There's should be at least one wave in a wave manager!");
        this.lastWaveOverTime = System.currentTimeMillis();
        this.waves = List.of(waves);
        this.addExtraDelay(DELAY_BETWEEN_WAVES);
    }
    
    public Wave getCurrentWave() {
        return waves.get(currentWave);
    }
    
    public int getCurrentWaveIndex() {
        return currentWave;
    }
    
    public boolean isVirusShouldBeGenerated() {
        return getCurrentWave().isVirusShouldBeCreated();
    }
    
    public boolean isCurrentWaveOver() {
        return getCurrentWave().isOver();
    }
    
    public AbstractVirus createVirus() {
        AbstractVirus virus = getCurrentWave().createVirus();
        if(isCurrentWaveOver()) lastWaveOverTime = System.currentTimeMillis();
        return virus;
    }
    
    public boolean hasNextWave() {
        return currentWave < waves.size() - 1;
    }
    
    public boolean isCurrentWaveLast() {
        return currentWave == waves.size() - 1;
    }
    
    public boolean isTimeBetweenWaves() {
        return lastWaveOverTime + DELAY_BETWEEN_WAVES > System.currentTimeMillis();
    }
    
    public void nextWave() {
        if(!hasNextWave()) throw new UnsupportedOperationException("Unable to start a new wave, because all of the waves are over");
        currentWave++;
    }
    
    public void addExtraDelay(long milliseconds) {
        getCurrentWave().addExtraDelay(milliseconds);
    }
    
    public static long getDelayBetweenWaves() { return DELAY_BETWEEN_WAVES; }
}
